/* Pinout of the SMS extension:
   BUFFER (8)			NOT USED
   B_8255 PORT A (8)	NOT USED
   B_8255 PORT B (8)	NOT USED
   B_8255 PORT Cl (4)	bit 0: /WR
						bit 1: /ZMREQ
						bit 2: /RD
						bit 3: PIN5
   B_8255 PORT Ch (4)	bit 0: PIN11
						bit 1: /MREQ
						bit 2: CLK
						bit 3: PIN49
   A_8255 PORT A (8)	ADDR 14-13-8-9-11-10-12-15
   A_8255 PORT B (8)	DATA 0..7
   A_8255 PORT Cl (4)	ADDR 7..4
   A_8255 PORT Ch (4)	ADDR 3..0
   GND,VCC... (8)
 */

#ifdef NAME
#undef NAME
#endif
#define NAME "SMS"

#define SMS_WR		0
#define SMS_RD		2
#define SMS_MREQ	5

#define SMS_IDLE	OUTPORT(B_8255_PC, 0xFF)
#define SMS_RD_ROM	OUTPORT(B_8255_PROG, CLR_PC(SMS_RD))

int sms_rom[] = {
	256, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, 32, 64, 128 };
char *sms_countries[] = {
	"?", "?", "?", "?", "?", "Japan", "Japan, USA, Europe", "USA, Europe",
	"?", "?", "?", "?", "?", "?", "?", "?" };

unsigned char *sms_header = NULL;

INLINE int sms_read_byte(register unsigned int addr)
{
	OUTPORT(A_8255_PC,
		((addr>>7)&0x01) |
		((addr>>5)&0x02) |
		((addr>>3)&0x04) |
		((addr>>1)&0x08) |
		((addr<<1)&0x10) |
		((addr<<3)&0x20) |
		((addr<<5)&0x40) |
		((addr<<7)&0x80));
	OUTPORT(A_8255_PA,
		((addr>>15)&0x01) |
		((addr>>11)&0x02) |
		((addr>>8 )&0x04) |
		((addr>>7 )&0x08) |
		((addr>>4 )&0x10) |
		((addr>>2 )&0x20) |
		((addr>>6 )&0x40) |
		((addr>>6 )&0x80));
	return INPORT(A_8255_PB);
}

void sms_read_header()
{
	unsigned int addr;

	SMS_RD_ROM;
	if(sms_header != NULL)
		return;

	sms_header = (unsigned char *)malloc(0x01 * sizeof(unsigned char));

	for(addr = 0x7FFF; addr < 0x8000; addr++)
		sms_header[addr-0x7FFF] = sms_read_byte(addr);
}

void sms_init()
{
	check_extension(NAME);

	/* Configure 8255's */
	/* B:
			port A: INPUT
			port B: INPUT
			port Cl: OUTPUT   0: /WR
							  1: /ZMREQ
							  2: /RD
							  3: PIN5
			port Ch: OUTPUT   0: PIN11
							  1: /MREQ
							  2: CLK
							  3: PIN49
	   A:
			port A: OUTPUT	  ADDR 14-13-8-9-11-10-12-15
			port B: INPUT	  DATA 0..7
			port Cl: OUTPUT	  ADDR 7..4
			port Ch: OUTPUT	  ADDR 3..0
	*/
	OUTPORT(B_8255_PROG, 0x80|PA_in|PB_in|PCl_out|PCh_out);
	OUTPORT(A_8255_PROG, 0x80|PA_out|PB_in|PCl_out|PCh_out);
	SMS_IDLE;
}

void sms_analyze()
{
	sms_read_header();

	printf("ROM size: %d(%d kB)\n", sms_header[0x00]&0x0F, sms_rom[sms_header[0x00]&0x0F]);
	printf("Country: %d(%s)\n", sms_header[0x00]>>4, sms_countries[sms_header[0x00]>>4]);
}

void sms_save(FILE *fp)
{
	unsigned char val;
	unsigned int addr;

	sms_analyze();

	for(addr = 0x0000; addr < 0x8000; addr++) {
		if(addr % 0x100 == 0) {
			putchar('.');
			fflush(stdout);
		}
		val = sms_read_byte(addr);
		fputc(val, fp);
	}
	putchar('\n');
}

void sms_debug()
{
	unsigned char val;
	unsigned int addr;

	SMS_RD_ROM;
	for(addr = 0x0001; ; addr <<= 1) {
		val = sms_read_byte(addr);
		printf("Addr = 0x%04X - Data = 0x%02X\n", addr, val);
		getchar();
		if(addr == 0x8000)
			break;
	}
}

Extension sms_ext = {
	NAME,
	sms_init,
	NULL,		/* no_flag */
	sms_analyze ,
	sms_save,
	NULL,		/* backup */
	NULL,		/* restore */
	NULL,		/* clear */
	NULL,		/* test */
	sms_debug,
	NULL,		/* extra */
	NULL		/* usage */
};
