/* Pinout of the VB extension:
   BUFFER (8)			NOT USED
   B_8255 PORT A (8)    ADDR NC-NC-NC-18-20-17-19-7
   B_8255 PORT B (8)    ADDR 8-6-9-5-10-4-11-3
   B_8255 PORT Cl (4)   ADDR 12-2-13-1
   B_8255 PORT Ch (4)   ADDR 14-0-15-16
   A_8255 PORT A (8)    DATA 15-0-7-8-14-1-6-9
   A_8255 PORT B (8)    DATA 13-2-5-10-12-3-4-11
   A_8255 PORT Cl (4)   bit 0: /RD
						bit 1: /POE
   A_8255 PORT Ch (4)   NOT USED
   GND,VCC... (8)
 */

#ifdef NAME
#undef NAME
#endif
#define NAME "VB"

#define VB_RD	0
#define VB_POE	1

#define VB_IDLE		OUTPORT(A_8255_PC, 0xFF)
#define VB_RD_ROM	OUTPORT(A_8255_PC, 0xFF-(0x01<<VB_RD)-(0x01<<VB_POE))

unsigned char *vb_header = NULL;

INLINE unsigned int vb_read_word(register unsigned long addr)
{
	register unsigned int i;

	OUTPORT(B_8255_PA,
		((addr>>15)&0x08) |
		((addr>>16)&0x10) |
		((addr>>12)&0x20) |
		((addr>>13)&0x40) |
		( addr     &0x80));
	OUTPORT(B_8255_PB,
		((addr>>8 )&0x01) |
		((addr>>5 )&0x02) |
		((addr>>7 )&0x04) |
		((addr>>2 )&0x08) |
		((addr>>6 )&0x10) |
		((addr<<1 )&0x20) |
		((addr>>5 )&0x40) |
		((addr<<4 )&0x80));
	OUTPORT(B_8255_PC,
		((addr>>12)&0x01) |
		((addr>>1 )&0x02) |
		((addr>>11)&0x04) |
		((addr<<2 )&0x08) |
		((addr>>10)&0x10) |
		((addr<<5 )&0x20) |
		((addr>>9 )&0x40) |
		((addr>>9 )&0x80));
	DELAY;
	i = INPORT(A_8255_PA) | (INPORT(A_8255_PB) << 8);
	return
		((i&0x0001)<<15) |
		((i&0x0002)>>1 ) |
		((i&0x0004)<<5 ) |
		((i&0x0008)<<5 ) |
		((i&0x0010)<<10) |
		((i&0x0020)>>4 ) |
		( i&0x0040     ) |
		((i&0x0080)<<2 ) |
		((i&0x0100)<<5 ) |
		((i&0x0200)>>7 ) |
		((i&0x0400)>>5 ) |
		((i&0x0800)>>1 ) |
		( i&0x1000     ) |
		((i&0x2000)>>10) |
		((i&0x4000)>>10) |
		((i&0x8000)>>4 );
}

void vb_read_header()
{
	unsigned int val, i;
	unsigned long addr;

	VB_RD_ROM;
	if(vb_header != NULL)
		return;

	vb_header = (unsigned char *)malloc(0x20 * sizeof(unsigned char));

	i = 0;
	for(addr = 0x7FEF0; addr < 0x7FF00; addr++) {
		val = vb_read_word(addr);
		vb_header[i++] = (unsigned char)val;
		vb_header[i++] = (unsigned char)(val>>8);
	}
}

void vb_init()
{
	check_extension(NAME);

	/* Configure 8255's */
	/* B:
			port A: OUTPUT    ADDR NC-NC-NC-18-20-17-19-7
			port B: OUTPUT    ADDR 8-6-9-5-10-4-11-3
			port Cl: OUTPUT   ADDR 12-2-13-1
			port Ch: OUTPUT   ADDR 14-0-15-16
	   A:
			port A: INPUT     DATA 15-0-7-8-14-1-6-9
			port B: INPUT     DATA 13-2-5-10-12-3-4-11
			port Cl: OUTPUT   bit 0: /RD
							  bit 1: /POE
			port Ch: INPUT    NOT USED
	*/
	OUTPORT(B_8255_PROG, 0x80|PA_out|PB_out|PCl_out|PCh_out);
	OUTPORT(A_8255_PROG, 0x80|PA_in|PB_in|PCl_out|PCh_in);
	VB_IDLE;
}

void vb_analyze()
{
	char name[21];

	vb_read_header();

	strncpy(name, &vb_header[0], 20);
	name[20] = 0;
	printf("Game name: %s\n", name);
}

void vb_save(FILE *fp)
{
	unsigned int val;
	unsigned long addr;
	unsigned int checksum;

	vb_analyze();

	checksum = 0;
	for(addr = 0x000000; addr < 0x100000; addr++) {
		if(addr % 0x1000 == 0) {
			putchar('.');
			fflush(stdout);
		}
		val = vb_read_word(addr);
		checksum += val;
		fputc(val, fp);
		fputc(val>>8, fp);
	}
	putchar('\n');
	printf("Checksum = 0x%04X\n", checksum);
}

void vb_debug()
{
	unsigned int val;
	unsigned long addr;

	VB_RD_ROM;
	for(addr = 0x000001; ; addr <<= 1) {
		val = vb_read_word(addr);
		printf("Addr = 0x%06lX - Data = 0x%04X\n", addr, val);
		getchar();
		if(addr == 0x100000)
			break;
	}
}

Extension vb_ext = {
	NAME,
	vb_init,
	NULL,		/* no_flag */
	vb_analyze ,
	vb_save,
	NULL,		/* backup */
	NULL,		/* restore */
	NULL,		/* clear */
	NULL,		/* test */
	vb_debug,
	NULL,		/* extra */
	NULL		/* usage */
};
